@@ -1,9 +1,9 @@
-/*  Lziprecover - Data recovery tool for lzipped files
-    Copyright (C) 2009, 2010, 2011, 2012 Antonio Diaz Diaz.
+/*  Lziprecover - Data recovery tool for the lzip format
+    Copyright (C) 2009-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -17,47 +17,30 @@
 
 class State
   {
-  unsigned char st;
+  int st;
 
 public:
   enum { states = 12 };
   State() : st( 0 ) {}
-  unsigned char operator()() const { return st; }
+  int operator()() const { return st; }
   bool is_char() const { return st < 7; }
 
   void set_char()
     {
-    static const unsigned char next[states] =
-      { 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 4, 5 };
-    st = next[st];
-    }
-
-  void set_match()
-    {
-    static const unsigned char next[states] =
-      { 7, 7, 7, 7, 7, 7, 7, 10, 10, 10, 10, 10 };
-    st = next[st];
-    }
-
-  void set_rep()
-    {
-    static const unsigned char next[states] =
-      { 8, 8, 8, 8, 8, 8, 8, 11, 11, 11, 11, 11 };
-    st = next[st];
-    }
-
-  void set_short_rep()
-    {
-    static const unsigned char next[states] =
-      { 9, 9, 9, 9, 9, 9, 9, 11, 11, 11, 11, 11 };
+    static const int next[states] = { 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 4, 5 };
     st = next[st];
     }
+  void set_char1()     { st -= ( st < 4 ) ? st : 3; }	// for st < 7
+  void set_char2()     { st -= ( st < 10 ) ? 3 : 6; }	// for st >= 7
+  void set_match()     { st = ( st < 7 ) ? 7 : 10; }
+  void set_rep()       { st = ( st < 7 ) ? 8 : 11; }
+  void set_short_rep() { st = ( st < 7 ) ? 9 : 11; }
   };
 
 
 enum {
   min_dictionary_bits = 12,
-  min_dictionary_size = 1 << min_dictionary_bits,
+  min_dictionary_size = 1 << min_dictionary_bits,	// >= modeled_distances
   max_dictionary_bits = 29,
   max_dictionary_size = 1 << max_dictionary_bits,
   min_member_size = 36,
@@ -66,10 +49,11 @@ enum {
   pos_states = 1 << pos_state_bits,
   pos_state_mask = pos_states - 1,
 
+  len_states = 4,
   dis_slot_bits = 6,
   start_dis_model = 4,
   end_dis_model = 14,
-  modeled_distances = 1 << (end_dis_model / 2),
+  modeled_distances = 1 << (end_dis_model / 2),		// 128
   dis_align_bits = 4,
   dis_align_size = 1 << dis_align_bits,
 
@@ -83,16 +67,13 @@ enum {
 
   min_match_len = 2,					// must be 2
   max_match_len = min_match_len + max_len_symbols - 1,	// 273
-  min_match_len_limit = 5,
+  min_match_len_limit = 5 };
 
-  max_dis_states = 4 };
+inline int get_len_state( const int len )
+  { return std::min( len - min_match_len, len_states - 1 ); }
 
-inline int get_dis_state( int len )
-  {
-  len -= min_match_len;
-  if( len >= max_dis_states ) len = max_dis_states - 1;
-  return len;
-  }
+inline int get_lit_state( const uint8_t prev_byte )
+  { return ( prev_byte >> ( 8 - literal_context_bits ) ); }
 
 
 enum { bit_model_move_bits = 5,
@@ -101,17 +82,26 @@ enum { bit_model_move_bits = 5,
 
 struct Bit_model
   {
-  unsigned int probability;
+  int probability;
   Bit_model() : probability( bit_model_total / 2 ) {}
   };
 
+struct Len_model
+  {
+  Bit_model choice1;
+  Bit_model choice2;
+  Bit_model bm_low[pos_states][len_low_symbols];
+  Bit_model bm_mid[pos_states][len_mid_symbols];
+  Bit_model bm_high[len_high_symbols];
+  };
+
 
 class Pretty_print
   {
-  const char * const stdin_name;
-  unsigned int longest_name;
-  const int verbosity_;
   std::string name_;
+  const char * const stdin_name;
+  unsigned longest_name;
+  const int verbosity_;
   mutable bool first_post;
 
 public:
@@ -119,11 +109,11 @@ public:
     : stdin_name( "(stdin)" ), longest_name( 0 ), verbosity_( v ),
       first_post( false )
     {
-    const unsigned int stdin_name_len = std::strlen( stdin_name );
-    for( unsigned int i = 0; i < filenames.size(); ++i )
+    const unsigned stdin_name_len = std::strlen( stdin_name );
+    for( unsigned i = 0; i < filenames.size(); ++i )
       {
       const std::string & s = filenames[i];
-      const unsigned int len = ( ( s == "-" ) ? stdin_name_len : s.size() );
+      const unsigned len = ( s == "-" ) ? stdin_name_len : s.size();
       if( len > longest_name ) longest_name = len;
       }
     if( longest_name == 0 ) longest_name = stdin_name_len;
@@ -132,8 +122,8 @@ public:
   Pretty_print( const std::string & filename, const int v )
     : stdin_name( "(stdin)" ), verbosity_( v ), first_post( false )
     {
-    const unsigned int stdin_name_len = std::strlen( stdin_name );
-    longest_name = ( ( filename == "-" ) ? stdin_name_len : filename.size() );
+    const unsigned stdin_name_len = std::strlen( stdin_name );
+    longest_name = ( filename == "-" ) ? stdin_name_len : filename.size();
     if( longest_name == 0 ) longest_name = stdin_name_len;
     set_name( filename );
     }
@@ -148,7 +138,7 @@ public:
   void reset() const { if( name_.size() ) first_post = true; }
   const char * name() const { return name_.c_str(); }
   int verbosity() const { return verbosity_; }
-  void operator()( const char * const msg = 0 ) const;
+  void operator()( const char * const msg = 0, FILE * const f = stderr ) const;
   };
 
 
@@ -159,9 +149,9 @@ class CRC32
 public:
   CRC32()
     {
-    for( unsigned int n = 0; n < 256; ++n )
+    for( unsigned n = 0; n < 256; ++n )
       {
-      unsigned int c = n;
+      unsigned c = n;
       for( int k = 0; k < 8; ++k )
         { if( c & 1 ) c = 0xEDB88320U ^ ( c >> 1 ); else c >>= 1; }
       data[n] = c;
@@ -169,9 +159,12 @@ public:
     }
 
   uint32_t operator[]( const uint8_t byte ) const { return data[byte]; }
-  void update( uint32_t & crc, const uint8_t byte ) const
+
+  void update_byte( uint32_t & crc, const uint8_t byte ) const
     { crc = data[(crc^byte)&0xFF] ^ ( crc >> 8 ); }
-  void update( uint32_t & crc, const uint8_t * const buffer, const int size ) const
+
+  void update_buf( uint32_t & crc, const uint8_t * const buffer,
+                   const int size ) const
     {
     for( int i = 0; i < size; ++i )
       crc = data[(crc^buffer[i])&0xFF] ^ ( crc >> 8 );
@@ -181,16 +174,15 @@ public:
 extern const CRC32 crc32;
 
 
-inline int real_bits( const unsigned int value )
+inline int real_bits( unsigned value )
   {
-  int bits = 0, i = 1;
-  unsigned int mask = 1;
-  for( ; mask > 0; ++i, mask <<= 1 ) if( value & mask ) bits = i;
+  int bits = 0;
+  while( value > 0 ) { value >>= 1; ++bits; }
   return bits;
   }
 
 
-const uint8_t magic_string[4] = { 'L', 'Z', 'I', 'P' };
+const uint8_t magic_string[4] = { 0x4C, 0x5A, 0x49, 0x50 };	// "LZIP"
 
 struct File_header
   {
@@ -204,25 +196,25 @@ struct File_header
     { return ( std::memcmp( data, magic_string, 4 ) == 0 ); }
 
   uint8_t version() const { return data[4]; }
-  bool verify_version() const { return ( data[4] <= 1 ); }
+  bool verify_version() const { return ( data[4] == 1 ); }
 
-  int dictionary_size() const
+  unsigned dictionary_size() const
     {
-    int sz = ( 1 << ( data[5] & 0x1F ) );
-    if( sz > min_dictionary_size && sz <= max_dictionary_size )
-      sz -= ( sz / 16 ) * ( ( data[5] >> 5 ) & 0x07 );
+    unsigned sz = ( 1 << ( data[5] & 0x1F ) );
+    if( sz > min_dictionary_size )
+      sz -= ( sz / 16 ) * ( ( data[5] >> 5 ) & 7 );
     return sz;
     }
 
-  bool dictionary_size( const int sz )
+  bool dictionary_size( const unsigned sz )
     {
     if( sz >= min_dictionary_size && sz <= max_dictionary_size )
       {
       data[5] = real_bits( sz - 1 );
       if( sz > min_dictionary_size )
         {
-        const int base_size = 1 << data[5];
-        const int wedge = base_size / 16;
+        const unsigned base_size = 1 << data[5];
+        const unsigned wedge = base_size / 16;
         for( int i = 7; i >= 1; --i )
           if( base_size - ( i * wedge ) >= sz )
             { data[5] |= ( i << 5 ); break; }
@@ -240,42 +232,37 @@ struct File_trailer
 			//  4-11 size of the uncompressed data
 			// 12-19 member size including header and trailer
 
-  static int size( const int version = 1 )
-    { return ( ( version >= 1 ) ? 20 : 12 ); }
+  enum { size = 20 };
 
-  uint32_t data_crc() const
+  unsigned data_crc() const
     {
-    uint32_t tmp = 0;
+    unsigned tmp = 0;
     for( int i = 3; i >= 0; --i ) { tmp <<= 8; tmp += data[i]; }
     return tmp;
     }
 
-  void data_crc( uint32_t crc )
+  void data_crc( unsigned crc )
     { for( int i = 0; i <= 3; ++i ) { data[i] = (uint8_t)crc; crc >>= 8; } }
 
-  long long data_size() const
+  unsigned long long data_size() const
     {
-    long long tmp = 0;
+    unsigned long long tmp = 0;
     for( int i = 11; i >= 4; --i ) { tmp <<= 8; tmp += data[i]; }
     return tmp;
     }
 
-  void data_size( long long sz )
-    {
-    for( int i = 4; i <= 11; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; }
-    }
+  void data_size( unsigned long long sz )
+    { for( int i = 4; i <= 11; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; } }
 
-  long long member_size() const
+  unsigned long long member_size() const
     {
-    long long tmp = 0;
+    unsigned long long tmp = 0;
     for( int i = 19; i >= 12; --i ) { tmp <<= 8; tmp += data[i]; }
     return tmp;
     }
 
-  void member_size( long long sz )
-    {
-    for( int i = 12; i <= 19; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; }
-    }
+  void member_size( unsigned long long sz )
+    { for( int i = 12; i <= 19; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; } }
   };
 
 
@@ -286,76 +273,56 @@ struct Error
   };
 
 
-#ifndef LLONG_MAX
-#define LLONG_MAX  0x7FFFFFFFFFFFFFFFLL
-#endif
-#ifndef LLONG_MIN
-#define LLONG_MIN  (-LLONG_MAX - 1LL)
-#endif
-#ifndef ULLONG_MAX
-#define ULLONG_MAX 0xFFFFFFFFFFFFFFFFULL
-#endif
-
-
-class Block
-  {
-  long long pos_, size_;		// pos + size <= LLONG_MAX
-
-public:
-  Block( const long long p, const long long s ) : pos_( p ), size_( s ) {}
-
-  long long pos() const { return pos_; }
-  long long size() const { return size_; }
-  long long end() const { return pos_ + size_; }
-
-  void pos( const long long p ) { pos_ = p; }
-  void size( const long long s ) { size_ = s; }
-
-  bool overlaps( const Block & b ) const
-    { return ( pos_ < b.end() && b.pos_ < end() ); }
-  void shift( Block & b ) { ++size_; ++b.pos_; --b.size_; }
-  };
+inline unsigned long long positive_diff( const unsigned long long x,
+                                         const unsigned long long y )
+  { return ( ( x > y ) ? x - y : 0 ); }
 
 
 // defined in decoder.cc
-int readblock( const int fd, uint8_t * const buf, const int size );
+long readblock( const int fd, uint8_t * const buf, const long size );
 int writeblock( const int fd, const uint8_t * const buf, const int size );
 
+// defined in file_index.cc
+int seek_read( const int fd, uint8_t * const buf, const int size,
+               const long long pos );
+
 // defined in main.cc
-extern int verbosity;
-const char * format_num( long long num, long long limit = LLONG_MAX,
-                         const int set_prefix = 0 );
-int open_instream( const std::string & name, struct stat * const in_statsp,
-                   const bool to_stdout, const bool reg_only = false );
-int open_outstream_rw( const std::string & output_filename,
-                       const bool force );
+int open_instream( const char * const name, struct stat * const in_statsp,
+                   const bool no_ofile, const bool reg_only = false );
+int open_outstream_rw( const std::string & output_filename, const bool force );
+void show_header( const File_header & header );
 void show_error( const char * const msg, const int errcode = 0,
                  const bool help = false );
+void show_error2( const char * const msg1, const char * const name,
+                  const char * const msg2 );
 void internal_error( const char * const msg );
 
 // defined in merge.cc
 void cleanup_and_fail( const std::string & output_filename,
                        const int outfd, const int retval );
 bool copy_file( const int infd, const int outfd,
-                const long long size = LLONG_MAX );
-bool try_decompress( const int fd, const long long file_size,
-                     long long * failure_posp = 0 );
-bool verify_header( const File_header & header );
-bool verify_single_member( const int fd, const long long file_size );
+                const long long max_size = -1 );
+bool try_decompress_member( const int fd, const unsigned long long msize,
+                            long long * failure_posp = 0 );
 int merge_files( const std::vector< std::string > & filenames,
-                 const std::string & output_filename, const bool force );
+                 const std::string & output_filename, const int verbosity,
+                 const bool force );
 
 // defined in range_dec.cc
-int list_file( const std::string & input_filename );
+bool safe_seek( const int fd, const long long pos );
+int list_files( const std::vector< std::string > & filenames,
+                const int verbosity );
 int range_decompress( const std::string & input_filename,
                       const std::string & default_output_filename,
-                      const std::string & range_string,
-                      const bool to_stdout, const bool force );
+                      const std::string & range_string, const int verbosity,
+                      const bool force, const bool ignore, const bool to_stdout );
 
 // defined in repair.cc
 int repair_file( const std::string & input_filename,
-                 const std::string & output_filename, const bool force );
+                 const std::string & output_filename, const int verbosity,
+                 const bool force );
 
 // defined in split.cc
 int split_file( const std::string & input_filename,
-                const std::string & default_output_filename, const bool force );
+                const std::string & default_output_filename,
+                const int verbosity, const bool force );
